## ----setup, echo=FALSE-----------------------------------------------------
suppressPackageStartupMessages({ 
    library(CNVRanger)
    library(AnnotationHub)
    library(regioneR)
    library(BSgenome.Btaurus.UCSC.bosTau6.masked)
    library(SummarizedExperiment)
    library(curatedTCGAData)
    library(TCGAutils)
})

## ----lib-------------------------------------------------------------------
library(CNVRanger)

## ----readCalls-------------------------------------------------------------
data.dir <- system.file("extdata", package="CNVRanger")
call.file <- file.path(data.dir, "Silva16_PONE_CNV_calls.csv")
calls <- read.csv(call.file, as.is=TRUE)
nrow(calls)
head(calls)

## ----nrSamples-------------------------------------------------------------
length(unique(calls[,"NE_id"]))

## ----cnvCalls--------------------------------------------------------------
grl <- makeGRangesListFromDataFrame(calls, 
    split.field="NE_id", keep.extra.columns=TRUE)
grl

## ----sortCalls-------------------------------------------------------------
grl <- sort(grl)
grl

## ----RaggedExperiment------------------------------------------------------
ra <- RaggedExperiment(grl)
ra

## ----RaggedExperiment-assay------------------------------------------------
assay(ra[1:5,1:5])

## ----RaggedExperiment-colData----------------------------------------------
weight <- rnorm(ncol(ra), mean=1100, sd=100)
fcr <- rnorm(ncol(ra), mean=6, sd=1.5)
colData(ra)$weight <- round(weight, digits=2)
colData(ra)$fcr <- round(fcr, digits=2)
colData(ra)

## ----cnvrs-----------------------------------------------------------------
cnvrs <- populationRanges(grl, density=0.1)
cnvrs

## ----cnvrsRO---------------------------------------------------------------
ro.cnvrs <- populationRanges(grl[1:100], mode="RO", ro.thresh=0.51)
ro.cnvrs

## ----gistic----------------------------------------------------------------
cnvrs <- populationRanges(grl, density=0.1, est.recur=TRUE)
cnvrs

## ----recurr----------------------------------------------------------------
cnvrs[cnvrs$pvalue < 0.05]

## ----getBtGenes------------------------------------------------------------
library(AnnotationHub)
ah <- AnnotationHub()
ahDb <- query(ah, pattern = c("Bos taurus", "EnsDb"))
ahDb

## ----getBtGenes2-----------------------------------------------------------
ahEdb <- ahDb[["AH60948"]]
bt.genes <- genes(ahEdb)
seqlevels(bt.genes) <- paste0("chr", seqlevels(bt.genes))
bt.genes

## ----formatBtGenes---------------------------------------------------------
sel.genes <- bt.genes[seqnames(bt.genes) %in% c("chr1", "chr2")]
sel.genes <- sel.genes[sel.genes$gene_biotype == "protein_coding"]
sel.cnvrs <- cnvrs[seqnames(cnvrs) %in% c("chr1", "chr2")]

## ----ovlpTest--------------------------------------------------------------
library(regioneR)
library(BSgenome.Btaurus.UCSC.bosTau6.masked)
res <- suppressWarnings(overlapPermTest(A=sel.cnvrs, B=sel.genes, ntimes=100, 
    genome="bosTau6", mask=NA, per.chromosome=TRUE, count.once=TRUE))
res

## ----permDist--------------------------------------------------------------
summary(res[[1]]$permuted)

## ----rseqdata--------------------------------------------------------------
rseq.file <- file.path(data.dir, "counts_cleaned.txt")
rcounts <- read.delim(rseq.file, row.names=1, stringsAsFactors=FALSE)
rcounts <- as.matrix(rcounts)
dim(rcounts)
rcounts[1:5, 1:5]

## ----rse-------------------------------------------------------------------
library(SummarizedExperiment)
rse <- SummarizedExperiment(assays=list(rcounts=rcounts), 
                rowRanges=granges(sel.genes)[rownames(rcounts)])
rse

## ----cnvEQTL---------------------------------------------------------------
res <- cnvEQTL(cnvrs, grl, rse, window="1Mbp", verbose=TRUE)
res

## ----tcgaSetup-------------------------------------------------------------
library(curatedTCGAData)
suppressMessages(
    gbm <- curatedTCGAData("GBM", 
        assays=c("GISTIC_Peaks", "CNVSNP", "RNASeq2GeneNorm"), 
        dry.run=FALSE)
)
gbm

## ----tcgaGeneAnno----------------------------------------------------------
library(TCGAutils)
gbm <- suppressMessages(symbolsToRanges(gbm, unmapped=FALSE))
for(i in 1:3) 
{
    genome(rowRanges(gbm[[i]])) <- "hg19"
    seqlevelsStyle(rowRanges(gbm[[i]])) <- "UCSC"
    ind <- as.character(seqnames(rowRanges(gbm[[i]]))) %in% c("chr1", "chr2")
    gbm[[i]] <- gbm[[i]][ind,]
}
gbm

## ----gbmIntersect----------------------------------------------------------
gbm <- intersectColumns(gbm)
sampleTables(gbm)
data(sampleTypes)
sampleTypes
gbm <- splitAssays(gbm, sampleCodes="01")
gbm

## ----transformToStates-----------------------------------------------------
ind <- grep("CNVSNP", names(gbm))
state <- round(mcols(gbm[[ind]])$Segment_Mean)
state[state > 2] <- 2
state[state < -2] <- -2
mcols(gbm[[ind]])$State <- state + 2
mcols(gbm[[ind]]) <- mcols(gbm[[ind]])[,3:1]
table(mcols(gbm[[ind]])$State)

## ----gbmEQTL---------------------------------------------------------------
res <- cnvEQTL(cnvrs="01_GBM_GISTIC_Peaks-20160128", 
    calls="01_GBM_CNVSNP-20160128", 
    rcounts="01_GBM_RNASeq2GeneNorm-20160128_ranged", 
    data=gbm, window="1Mbp", verbose=TRUE)
res

## ----readCalls2------------------------------------------------------------
cnv.loc <- file.path(data.dir, "CNVOut.txt") 
cnv.calls <- read.delim(cnv.loc, as.is=TRUE) 
cnv.calls <- makeGRangesListFromDataFrame(cnv.calls, 
    split.field="sample.id", keep.extra.columns=TRUE)
cnv.calls

## ----importPhen------------------------------------------------------------
## GEBV values
phen.loc <- file.path(data.dir, "Pheno.txt")

## Genomic positions of the probes used in the CNV call
map.loc <- file.path(data.dir, "MapPenn.txt")

phen.info <- setupCnvGWAS("example", phen.loc, cnv.calls, map.loc)

## ----importPhenRagged------------------------------------------------------
## Phenotypes and CNVs can be also be imported as a RaggedExperiment object
phen.df <- read.delim(phen.loc)
colDat <- DataFrame(phen.df)
re <- RaggedExperiment(cnv.calls, colData=colDat)
re
phen.info <- setupCnvGWAS("example", cnv.out.loc=re, map.loc=map.loc)
phen.info

## ----Wdir------------------------------------------------------------------
all.paths <- phen.info$all.paths
all.paths

## ----CNVGWASNames----------------------------------------------------------
# Define chr correspondence to numeric
chr.code.name<- data.frame(   
                    V1=c(16, 25, 29:31), 
                    V2=c("1A", "4A", "25LG1", "25LG2", "LGE22"))

## ----CNVGWA----------------------------------------------------------------
segs.pvalue.gr <- cnvGWAS(phen.info, chr.code.name=chr.code.name, method.m.test="none")
segs.pvalue.gr

## ----manh------------------------------------------------------------------
## Define the chromosome order in the plot
order.chrs <- c(1:24, "25LG1", "25LG2", 27:28, "LGE22", "1A", "4A")

## Chromosome sizes
chr.size.file <- file.path(data.dir, "Parus_major_chr_sizes.txt")
chr.sizes <- scan(chr.size.file)
chr.size.order <- data.frame(chr=order.chrs, sizes=chr.sizes, stringsAsFactors=FALSE)

## Plot a pdf file with a manhatthan within the 'Results' workfolder
plotManhattan(all.paths, segs.pvalue.gr, chr.size.order, plot.pdf=FALSE)

## ----importLRR-------------------------------------------------------------
# List files to import LRR/BAF 
files <- list.files(data.dir, pattern = "\\.cnv.txt.adjusted$")
samples <- sub(".cnv.txt.adjusted$", "", files)
samples <- sub("^GT","", samples)
sample.files <- data.frame(file.names=files, sample.names=samples)
 
# All missing samples will have LRR = '0' and BAF = '0.5' in all SNPs listed in the GDS file
importLrrBaf(all.paths, data.dir, sample.files, verbose=FALSE)

# Read the GDS to check if the LRR/BAF nodes were added
cnv.gds <- file.path(all.paths[1], "CNV.gds")
(genofile <- SNPRelate::snpgdsOpen(cnv.gds, allow.fork=TRUE, readonly=FALSE))
gdsfmt::closefn.gds(genofile)

# Run the CNV-GWAS with existent GDS
segs.pvalue.gr <- cnvGWAS(phen.info, chr.code.name=chr.code.name, produce.gds=FALSE, run.lrr=TRUE)

## ----prodGDS---------------------------------------------------------------
## Create a GDS file in disk and export the SNP probe positions
probes.cnv.gr <- generateGDS(phen.info, chr.code.name=chr.code.name)
probes.cnv.gr

## Run GWAS with existent GDS file
segs.pvalue.gr <- cnvGWAS(phen.info, chr.code.name=chr.code.name, produce.gds=FALSE)

## ----sessionInfo-----------------------------------------------------------
sessionInfo()

